/*
 * Copyright (c) 2004-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef PORT_DATA_H_
#define PORT_DATA_H_

#include "agg_types.h"
#include "agg_node.h"

class PortData;
class HostInfo;

struct PortDataSort {
    inline bool operator() (const PortData *p_lhs, const PortData *p_rhs) const;
};

typedef std::set< const PortData *, PortDataSort > SetPortDataConstPtr;
typedef std::vector< const PortData * > VecPortDataConstPtr;
typedef std::map< port_key_t, class PortData * > MapPortDataPtr;

enum PortType {
    CA_PORT_TYPE_UNKNOWN = 0,
    CA_PORT_TYPE_COMPUTE = 1,
    CA_PORT_TYPE_AGG_NODE = 2,
    CA_PORT_TYPE_UNSUPPORTED_AGG_NODE = 3,
    CA_VPORT_TYPE = 4,
    CA_PORT_TYPE_OTHER = 5
};

static inline const char * CaPortType2Char(const PortType port_type)
{
    switch (port_type) {
    case CA_PORT_TYPE_UNKNOWN:    return("UNKNOWN");
    case CA_PORT_TYPE_COMPUTE:    return("COMPUTE");
    case CA_PORT_TYPE_AGG_NODE:   return("AN");
    case CA_PORT_TYPE_UNSUPPORTED_AGG_NODE:   return("UNSUPPORTED_AN");
    case CA_VPORT_TYPE:           return("VIRTUAL");
    default:            return("INVALID");
    }
};

class PortData {
protected:
    PortInfo            m_port_info_;
    uint64_t            m_epoch_;
    PortType            m_port_type_;
    AggNode            *m_agg_node_;
    const HostInfo     *m_host_info_;

    //prevent usage of copy ctor and copy ctor and assignment operator
    //PortData is the owner of m_pAggNode
    PortData(const PortData& other);
    const PortData& operator=(const PortData& rhs);

public:
    PortData(const PortInfo &port_info, uint64_t epoch):
        m_port_info_(port_info), m_epoch_(epoch),
        m_port_type_(CA_PORT_TYPE_UNKNOWN), m_agg_node_(NULL),
        m_host_info_(NULL) {}

    virtual ~PortData() {
        if (m_agg_node_)
            delete m_agg_node_;
    }

    void SetType(PortType port_type) { m_port_type_ = port_type; }
    PortType GetType() { return m_port_type_; }

    const PortInfo &GetPortInfo() const { return m_port_info_; }
    virtual const PortInfo &GetPhysPortInfo() const;
    void SetPortInfo(const PortInfo & port_info) { m_port_info_ = port_info; }

    void SetTimestamp(const PortTimestamp & timestamp) {
        m_port_info_.m_timestamp = timestamp;
    }

    void SetPortInfoPeerKey(const port_key_t & m_peer_key) {
        m_port_info_.m_peer_key = m_peer_key;
    }

    void SetPortInfoLid(const lid_t & lid) {
        m_port_info_.m_lid = lid;
    }

    void SetAggNode(AggNode *p_agg_node) {
        m_agg_node_ = p_agg_node;
        m_port_type_ = CA_PORT_TYPE_AGG_NODE;
    }
    AggNode *GetAggNode() { return m_agg_node_; }

    const HostInfo *GetHostInfo() const { return m_host_info_; }
    //sharpd_id_t GetSharpdId() const { return m_host_info_->GetSharpdId(); }

    void SetHostInfo(const HostInfo *p_host_info) {
        m_host_info_ = p_host_info;
    }

    string ToString() const;
    virtual void Update(const PortInfo &port_info);
};

class VportData : public PortData {

    PortInfo m_phys_port_info_;

public:
    VportData(const PortInfo &phys_port_info, const PortInfo &vport_info,
              uint64_t epoch) : PortData(vport_info, epoch),
                                m_phys_port_info_(phys_port_info)
    {
        m_port_type_ = CA_VPORT_TYPE;
    }
    virtual ~VportData();

    virtual const PortInfo &GetPhysPortInfo() const;

    virtual void Update(const PortInfo &vport_info);
};

bool PortDataSort::operator()(const PortData *p_lhs,
                              const PortData *p_rhs) const {
    return (p_lhs->GetPortInfo().m_port_key < p_rhs->GetPortInfo().m_port_key);
}

#endif //PORT_DATA_H_
